@@ -1,5 +1,5 @@
 
-#  $Id: Pg.pm,v 1.58 2003/09/08 21:39:41 markjugg Exp $
+#  $Id: Pg.pm,v 1.65 2003/10/29 21:22:12 rlippan Exp $
 #
 #  Copyright (c) 1997,1998,1999,2000 Edmund Mergl
 #  Copyright (c) 2002 Jeffrey W. Baker
@@ -12,7 +12,7 @@
 
 use 5.006001;
 
-$DBD::Pg::VERSION = '1.31_5';
+$DBD::Pg::VERSION = '1.31_7';
 
 {
 	package DBD::Pg;
@@ -189,148 +189,88 @@ $DBD::Pg::VERSION = '1.31_5';
 	# column_size, buffer_length, DECIMAL_DIGITS, NUM_PREC_RADIX, NULLABLE,
 	# REMARKS, COLUMN_DEF, SQL_DATA_TYPE, SQL_DATETIME_SUB, CHAR_OCTET_LENGTH,
 	# ORDINAL_POSITION, IS_NULLABLE
-	# The result set is ordered by TABLE_CAT, TABLE_SCHEM, 
-	# TABLE_NAME and ORDINAL_POSITION.
+	# The result set is ordered by TABLE_SCHEM, TABLE_NAME and ORDINAL_POSITION.
 
 	sub column_info {
-		my ($dbh) = shift;
-		my @attrs = @_;
-		# my ($dbh, $catalog, $schema, $table, $column) = @_;
-		my $CATALOG = DBD::Pg::_pg_use_catalog($dbh);
+		my $dbh = shift;
+		my ($catalog, $schema, $table, $column) = @_;
 
+		my $CATALOG = DBD::Pg::_pg_use_catalog($dbh);
 		my $version = DBD::Pg::_pg_server_version($dbh);
 
-		my @wh = ();
-		my @flds = qw/catname n.nspname c.relname a.attname/;
-
-		for my $idx (0 .. $#attrs) {
-			next if $flds[$idx] eq 'catname'; # Skip catalog
-			next if $flds[$idx] eq 'n.nspname' and ! DBD::Pg::_pg_check_version(7.3, $version);
-			if(defined $attrs[$idx] and length $attrs[$idx]) {
-				# Insure that the value is enclosed in single quotes.
-				$attrs[$idx] =~ s/^'?(\w+)'?$/'$1'/;
-				if ($attrs[$idx] =~ m/[,%]/) {
-					# contains a meta character.
-					push( @wh, q{( } . join ( " OR "
-						, map { m/\%/ 
-							? qq{$flds[$idx] ILIKE $_ }
-							: qq{$flds[$idx] = $_ }
-							} (split /,/, $attrs[$idx]) )
-							. q{ )}
-						);
-				}
-				else {
-					push( @wh, qq{$flds[$idx] = $attrs[$idx]} );
-				}
-			}
+		my @search;
+		## If the schema or table has an underscore or a %, use a LIKE comparison
+		if (defined $schema and length $schema and DBD::Pg::_pg_check_version(7.3, $version)) {
+			push @search, "n.nspname " . ($schema =~ /[_%]/ ? "LIKE " : "= ") . 
+				$dbh->quote($schema);
+		}
+		if (defined $table and length $table) {
+			push @search, "c.relname " . ($table =~ /[_%]/ ? "LIKE " : "= ") .
+				$dbh->quote($table);
+		}
+		if (defined $column and length $column) {
+			push @search, "a.attname " . ($column =~ /[_%]/ ? "LIKE " : "= ") .
+				$dbh->quote($column);
 		}
 
-		my $wh = ""; # ();
-		$wh = join( " AND ", '', @wh ) if (@wh);
-		# my $showschema = DBD::Pg::_pg_check_version(7.3, $version) ? 
-		# 	"n.nspname" : "NULL::text";
+		my $whereclause = join "\n\t\t\t\tAND ", "", @search;
 
-		# my $schemajoin = DBD::Pg::_pg_check_version(7.3, $version) ? 
-		# 	"LEFT JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace)" : "";
+		my $showschema = DBD::Pg::_pg_check_version(7.3, $version) ? 
+			"n.nspname" : "NULL::text";
 
-		# my $pg_description_join  =  DBD::Pg::_pg_check_version(7.2, $version) ?
-		# 	"  LEFT JOIN ${CATALOG}pg_description d  ON (a.attnum = d.objsubid)
-		# 	   LEFT JOIN ${CATALOG}pg_class       dc ON (dc.oid = d.objoid ) "
-		# 	:	
-		# 	"   LEFT JOIN ${CATALOG}pg_description d ON (a.oid = d.objoid )";
+		my $schemajoin = DBD::Pg::_pg_check_version(7.3, $version) ? 
+			"JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace)" : "";
 
-		my $col_info_sql;
-
-		if (DBD::Pg::_pg_check_version( 7.3, $version) < 2) {
-			$col_info_sql = qq!
+		my $pg_description_join = DBD::Pg::_pg_check_version(7.2, $version) ?
+		 	"LEFT JOIN ${CATALOG}pg_description d  ON (a.attnum = d.objsubid)\n".
+				"\t\t\t\tLEFT JOIN ${CATALOG}pg_class       dc ON (dc.oid = d.objoid )"
+					:
+						"LEFT JOIN ${CATALOG}pg_description d ON (a.oid = d.objoid )";
+		
+		my $col_info_sql = qq!
 			SELECT
-					NULL::text AS "TABLE_CAT"
-				, NULL::text AS "TABLE_SCHEM"
+				NULL::text AS "TABLE_CAT"
+				, $showschema AS "TABLE_SCHEM"
 				, c.relname AS "TABLE_NAME"
 				, a.attname AS "COLUMN_NAME"
-				, a.atttypid 	AS "DATA_TYPE"
-				, format_type(a.atttypid, NULL)	as "TYPE_NAME"
+				, a.atttypid AS "DATA_TYPE"
+				, ${CATALOG}format_type(a.atttypid, NULL) AS "TYPE_NAME"
 				, a.attlen AS "COLUMN_SIZE"
 				, NULL::text AS "BUFFER_LENGTH"
 				, NULL::text AS "DECIMAL_DIGITS"
 				, NULL::text AS "NUM_PREC_RADIX"
-				, CASE a.attnotnull WHEN 't' THEN 0 ELSE 1 END  AS "NULLABLE"
-				, col_description(a.attrelid, a.attnum) AS "REMARKS"
+				, CASE a.attnotnull WHEN 't' THEN 0 ELSE 1 END AS "NULLABLE"
+				, ${CATALOG}col_description(a.attrelid, a.attnum) AS "REMARKS"
 				, af.adsrc AS "COLUMN_DEF"
 				, NULL::text AS "SQL_DATA_TYPE"
 				, NULL::text AS "SQL_DATETIME_SUB"
 				, NULL::text AS "CHAR_OCTET_LENGTH"
 				, a.attnum AS "ORDINAL_POSITION"
-				, CASE a.attnotnull WHEN 't' THEN 'NO' ELSE 'YES' END  AS "IS_NULLABLE"
-				, format_type(a.atttypid, a.atttypmod)	as "pg_type"
-				, format_type(a.atttypid, NULL)	as "pg_type_only"
-				, a.atttypmod    AS "pg_atttypmod"
-			FROM
-				pg_type t
-				LEFT JOIN pg_attribute a
-					ON (t.oid = a.atttypid)
-				LEFT JOIN pg_class c
-					ON (a.attrelid = c.oid)
-				LEFT JOIN pg_attrdef af
-					ON (a.attnum = af.adnum AND a.attrelid = af.adrelid)
-			WHERE
-									a.attnum >= 0
-							AND c.relkind IN ('r','v')
-							$wh
-			ORDER BY 2, c.relname, a.attnum
-			!;
-		} else {
-			$col_info_sql = qq!
-			SELECT
-				  NULL::text	AS "TABLE_CAT"
-				, n.nspname		AS "TABLE_SCHEM"
-				, c.relname		AS "TABLE_NAME"
-				, a.attname		AS "COLUMN_NAME"
-				, a.atttypid 	AS "DATA_TYPE"
-				, format_type(a.atttypid, NULL)	as "TYPE_NAME"
-				, a.attlen		AS "COLUMN_SIZE"
-				, NULL::text	AS "BUFFER_LENGTH"
-				, NULL::text	AS "DECIMAL_DIGITS"
-				, NULL::text	AS "NUM_PREC_RADIX"
-				, CASE a.attnotnull WHEN 't' THEN 0 ELSE 1 END AS "NULLABLE"
-				, ${CATALOG}col_description(a.attrelid, a.attnum) AS "REMARKS"
-				, pg_attrdef.adsrc	AS "COLUMN_DEF"
-				, NULL::text	AS "SQL_DATA_TYPE"
-				, NULL::text	AS "SQL_DATETIME_SUB"
-				, NULL::text	AS "CHAR_OCTET_LENGTH"
-				, a.attnum		AS "ORDINAL_POSITION"
 				, CASE a.attnotnull WHEN 't' THEN 'NO' ELSE 'YES' END AS "IS_NULLABLE"
-				, format_type(a.atttypid, a.atttypmod)	as "pg_type"
-				, format_type(a.atttypid, NULL)	as "pg_type_only"
-				, a.atttypmod   AS "pg_atttypmod"
-			FROM 
-				  ${CATALOG}pg_type		t
-				, ${CATALOG}pg_class c
-				  LEFT JOIN ${CATALOG}pg_namespace n 
-				  	ON (n.oid = c.relnamespace)
-				, ${CATALOG}pg_attribute	a
-		    		  LEFT JOIN ${CATALOG}pg_attrdef 
-				  	ON (a.attnum = pg_attrdef.adnum 
-						AND a.attrelid = pg_attrdef.adrelid) 
+				, ${CATALOG}format_type(a.atttypid, a.atttypmod) AS "pg_type"
+				, ${CATALOG}format_type(a.atttypid, NULL) AS "pg_type_only"
+				, a.atttypmod AS "pg_atttypmod"
+			FROM
+				${CATALOG}pg_type t
+				JOIN ${CATALOG}pg_attribute a ON (t.oid = a.atttypid)
+				JOIN ${CATALOG}pg_class c ON (a.attrelid = c.oid)
+				LEFT JOIN ${CATALOG}pg_attrdef af ON (a.attnum = af.adnum AND a.attrelid = af.adrelid)
+				$schemajoin
+				$pg_description_join
 			WHERE
-					a.attrelid = c.oid
-				AND a.attnum >= 0
-				AND t.oid = a.atttypid
+				a.attnum >= 0
 				AND c.relkind IN ('r','v')
-				$wh
-			ORDER BY 2, c.relname, a.attnum
+				$whereclause
+			ORDER BY "TABLE_SCHEM", "TABLE_NAME", "ORDINAL_POSITION"
 			!;
 
-		}
-
-		my $data = $dbh->selectall_arrayref($col_info_sql) || return undef;
-
+		my $data = $dbh->selectall_arrayref($col_info_sql) or return undef;
 
 		# To turn the data back into a statement handle, we need 
 		# to fetch the data as an array of arrays, and also have a
 		# a matching array of all the column names
 		my %col_map = (qw/
-			TABLE_CAT			  0 
+			TABLE_CAT             0 
 			TABLE_SCHEM           1 
 			TABLE_NAME            2 
 			COLUMN_NAME           3 
@@ -352,18 +292,21 @@ $DBD::Pg::VERSION = '1.31_5';
 			pg_type_only				 19
 			pg_atttypmod				 20
 			/);
-
-		 my $constraint_query = DBD::Pg::_pg_check_version(7.3, $version)
-			 ? "SELECT consrc FROM pg_catalog.pg_constraint WHERE contype = 'c' AND conname = ?" 
-			 : "SELECT rcsrc FROM pg_relcheck WHERE rcname = ?";
-	     my $constraint_sth = $dbh->prepare($constraint_query); 		 
+		
+		my $constraint_query = DBD::Pg::_pg_check_version(7.3, $version)
+			? "SELECT consrc FROM pg_catalog.pg_constraint WHERE contype = 'c' AND conname = ?" 
+				: "SELECT rcsrc FROM pg_relcheck WHERE rcname = ?";
+		my $constraint_sth = $dbh->prepare($constraint_query); 		 
 
 		for my $row (@$data) {
-			$row->[$col_map{COLUMN_SIZE}]   = _calc_col_size($row->[$col_map{pg_atttypmod}],$row->[$col_map{COLUMN_SIZE}]);
+			$row->[$col_map{COLUMN_SIZE}] = 
+ 				_calc_col_size($row->[$col_map{pg_atttypmod}],$row->[$col_map{COLUMN_SIZE}]);
 
 			# Replace the Pg type with the SQL_ type
-			$row->[$col_map{DATA_TYPE}]     = DBD::Pg::db::pg_type_info($dbh,$col_map{DATA_TYPE});
-
+			my $w = $row->[$col_map{DATA_TYPE}];
+			$row->[$col_map{DATA_TYPE}] = DBD::Pg::db::pg_type_info($dbh,$row->[$col_map{DATA_TYPE}]);
+			$w = $row->[$col_map{DATA_TYPE}];
+			
 			pop @$row;
 
 			# Add pg_constraint
@@ -381,10 +324,7 @@ $DBD::Pg::VERSION = '1.31_5';
 		my $sth = _prepare_from_data(
 			'column_info', 
 			$data, 
-		    [ sort { $col_map{$a} <=> $col_map{$b}  } keys %col_map]);
-
-
-		return $sth;
+				[ sort { $col_map{$a} <=> $col_map{$b}  } keys %col_map]);
 	}
 
 	sub _prepare_from_data {
@@ -734,243 +674,137 @@ $DBD::Pg::VERSION = '1.31_5';
 	sub table_info {	# DBI spec: TABLE_CAT, TABLE_SCHEM, TABLE_NAME, TABLE_TYPE, REMARKS
 		my $dbh = shift;
 		my ($catalog, $schema, $table, $type) = @_;
-		my @attrs = @_;
 
 		my $tbl_sql = ();
 
 		my $version = DBD::Pg::_pg_server_version($dbh);
 		my $CATALOG = DBD::Pg::_pg_use_catalog($dbh);
-		my $schemacase = DBD::Pg::_pg_check_version(7.3, $version) ? 
-			"CASE WHEN n.nspname ~ '^pg_' THEN 'SYSTEM TABLE' ELSE 'TABLE' END" : 
-			"CASE WHEN c.relname ~ '^pg_' THEN 'SYSTEM TABLE' ELSE 'TABLE' END";
 
-		if ( # Rules 19a
+		if ( # Rule 19a
 				(defined $catalog and $catalog eq '%')
-			and (defined $schema and $schema eq '')
-			and (defined $table  and $table  eq '')
-			) {
-				$tbl_sql = q{
+				and (defined $schema and $schema eq '')
+				and (defined $table and $table eq '')
+			 ) {
+			$tbl_sql = q{
 					SELECT 
-					   NULL::text AS "TABLE_CAT"
+						 NULL::text AS "TABLE_CAT"
 					 , NULL::text AS "TABLE_SCHEM"
 					 , NULL::text AS "TABLE_NAME"
 					 , NULL::text AS "TABLE_TYPE"
 					 , NULL::text AS "REMARKS"
 					};
 		}
-		elsif (# Rules 19b
-				(defined $catalog and $catalog eq '')
-			and (defined $schema and $schema eq '%')
-			and (defined $table  and $table  eq '')
-			) {
-				$tbl_sql = DBD::Pg::_pg_check_version(7.3, $version) ? 
-					q{SELECT 
-					   NULL::text AS "TABLE_CAT"
+		elsif (# Rule 19b
+					 (defined $catalog and $catalog eq '')
+					 and (defined $schema and $schema eq '%')
+					 and (defined $table and $table eq '')
+					) {
+			$tbl_sql = DBD::Pg::_pg_check_version(7.3, $version) ? 
+				q{SELECT 
+						 NULL::text AS "TABLE_CAT"
 					 , n.nspname  AS "TABLE_SCHEM"
 					 , NULL::text AS "TABLE_NAME"
 					 , NULL::text AS "TABLE_TYPE"
-					 , NULL::text AS "REMARKS"
+					 , CASE WHEN n.nspname ~ '^pg_' THEN 'system schema' ELSE 'owned by ' || pg_get_userbyid(n.nspowner) END AS "REMARKS"
 					FROM pg_catalog.pg_namespace n
-					ORDER BY 1
+					ORDER BY "TABLE_SCHEM"
 					} : 
-					q{SELECT 
-					   NULL::text AS "TABLE_CAT"
+						q{SELECT 
+						 NULL::text AS "TABLE_CAT"
 					 , NULL::text AS "TABLE_SCHEM"
 					 , NULL::text AS "TABLE_NAME"
 					 , NULL::text AS "TABLE_TYPE"
 					 , NULL::text AS "REMARKS"
 				};
 		}
-		elsif (# Rules 19c
-				(defined $catalog and $catalog eq '')
-			and (defined $schema and $schema eq '')
-			and (defined $table  and $table  eq '')
-			and (defined $type   and $type   eq '%')
-			) {
-				# From the postgresql 7.2.1 manual 3.5 pg_class
-				#  'r' = ordinary table
-				#, 'i' = index
-				#, 'S' = sequence
-				#, 'v' = view
-				#, 's' = special
-				#, 't' = secondary TOAST table 
-				$tbl_sql = q{
+		elsif (# Rule 19c
+					 (defined $catalog and $catalog eq '')
+					 and (defined $schema and $schema eq '')
+					 and (defined $table and $table eq '')
+					 and (defined $type and $type eq '%')
+					) {
+			$tbl_sql = q{
 					SELECT 
-					   NULL::text    AS "TABLE_CAT"
-					 , NULL::text    AS "TABLE_SCHEM"
-					 , NULL::text    AS "TABLE_NAME"
-					 , 'table'       AS "TABLE_TYPE"
-					 , 'ordinary table - r' AS "REMARKS"
+					   NULL::text AS "TABLE_CAT"
+					 , NULL::text AS "TABLE_SCHEM"
+					 , NULL::text AS "TABLE_NAME"
+					 , 'TABLE'    AS "TABLE_TYPE"
+					 , 'relkind: r' AS "REMARKS"
 					UNION
 					SELECT 
-					   NULL::text    AS "TABLE_CAT"
-					 , NULL::text    AS "TABLE_SCHEM"
-					 , NULL::text    AS "TABLE_NAME"
-					 , 'index'       AS "TABLE_TYPE"
-					 , 'index - i'   AS "REMARKS"
-					UNION
-					SELECT 
-					   NULL::text     AS "TABLE_CAT"
-					 , NULL::text     AS "TABLE_SCHEM"
-					 , NULL::text     AS "TABLE_NAME"
-					 , 'sequence'     AS "TABLE_TYPE"
-					 , 'sequence - S' AS "REMARKS"
-					UNION
-					SELECT 
-					   NULL::text     AS "TABLE_CAT"
-					 , NULL::text     AS "TABLE_SCHEM"
-					 , NULL::text     AS "TABLE_NAME"
-					 , 'view'         AS "TABLE_TYPE"
-					 , 'view - v'     AS "REMARKS"
-					UNION
-					SELECT 
-					   NULL::text     AS "TABLE_CAT"
-					 , NULL::text     AS "TABLE_SCHEM"
-					 , NULL::text     AS "TABLE_NAME"
-					 , 'special'      AS "TABLE_TYPE"
-					 , 'special - s'  AS "REMARKS"
-					UNION
-					SELECT 
-					   NULL::text     AS "TABLE_CAT"
-					 , NULL::text     AS "TABLE_SCHEM"
-					 , NULL::text     AS "TABLE_NAME"
-					 , 'secondary'    AS "TABLE_TYPE"
-					 , 'secondary TOAST table - t' AS "REMARKS"
+					   NULL::text AS "TABLE_CAT"
+					 , NULL::text AS "TABLE_SCHEM"
+					 , NULL::text AS "TABLE_NAME"
+					 , 'VIEW'     AS "TABLE_TYPE"
+					 , 'relkind: v' AS "REMARKS"
 				};
 		}
 		else {
 			# Default SQL
-			my $showschema = DBD::Pg::_pg_check_version(7.3, $version) ? "n.nspname" : "NULL::text";
-			my $schemajoin = DBD::Pg::_pg_check_version(7.3, $version) ? 
-				"LEFT JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace)" : "";
-			my $has_objsubid = DBD::Pg::_pg_check_version(7.2, $version) ? 
-				"AND d.objsubid = 0" : "";
+			my $showschema = "NULL::text";
+			my $schemajoin = "";
+			my $has_objsubid = "";
+			if (DBD::Pg::_pg_check_version(7.3, $version)) {
+				$showschema = "n.nspname";
+				$schemajoin = "LEFT JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace)";
+				$has_objsubid = "AND d.objsubid = 0";
+			}
+
+			my @search;
+			## If the schema or table has an underscore or a %, use a LIKE comparison
+			if (defined $schema and length $schema and DBD::Pg::_pg_check_version(7.3, $version)) {
+					push @search, "n.nspname " . ($schema =~ /[_%]/ ? "LIKE " : "= ") . $dbh->quote($schema);
+			}
+			if (defined $table and length $table) {
+					push @search, "c.relname " . ($table =~ /[_%]/ ? "LIKE " : "= ") . $dbh->quote($table);
+			}
+			## All we can see is "table" or "view". Default is both
+			my $typesearch = "IN ('r','v')";
+			if (defined $type and length $type) {
+					if ($type =~ /\btable\b/i and $type !~ /\bview\b/i) {
+							$typesearch = "= 'r'";
+					}
+					elsif ($type =~ /\bview\b/i and $type !~ /\btable\b/i) {
+							$typesearch = "= 'v'";
+					}
+			}
+			push @search, "c.relkind $typesearch";
+			
+			my $whereclause = join "\n\t\t\t\t\t AND " => @search;
+			my $schemacase = DBD::Pg::_pg_check_version(7.3, $version) ? "n.nspname" : "c.relname";
 			$tbl_sql = qq{
 				SELECT NULL::text AS "TABLE_CAT"
-					 , $showschema  AS "TABLE_SCHEM"
-					 , c.relname    AS "TABLE_NAME"
+					 , $showschema AS "TABLE_SCHEM"
+					 , c.relname AS "TABLE_NAME"
 					 , CASE
-					 	WHEN c.relkind = 'v' THEN 'VIEW'
-						ELSE $schemacase
+					 		WHEN c.relkind = 'v' THEN
+								CASE WHEN $schemacase ~ '^pg_' THEN 'SYSTEM VIEW' ELSE 'VIEW' END
+							ELSE
+								CASE WHEN $schemacase ~ '^pg_' THEN 'SYSTEM TABLE' ELSE 'TABLE' END
 						END AS "TABLE_TYPE"
 					 , d.description AS "REMARKS"
 				FROM ${CATALOG}pg_class AS c
-					LEFT JOIN 
-					${CATALOG}pg_description AS d 
+					LEFT JOIN ${CATALOG}pg_description AS d 
 						ON (c.relfilenode = d.objoid $has_objsubid)
 					$schemajoin
-				WHERE 
-					((c.relkind = 'r'
-				 AND c.relhasrules = FALSE) OR
-					(c.relkind = 'v'
-				 AND c.relhasrules = TRUE))
-				 AND c.relname !~ '^xin[vx][0-9]+'
-				ORDER BY 1, 2, 3
+				WHERE $whereclause
+				ORDER BY "TABLE_TYPE", "TABLE_CAT", "TABLE_SCHEM", "TABLE_NAME"
 				};
-
-			# Did we receive any arguments?
-			if (@attrs) {
-				my @wh = ();
-				my @flds = qw/catname n.nspname c.relname c.relkind/;
-
-				for my $idx (0 .. $#attrs) {
-					next if $flds[$idx] eq 'catname'; # Skip catalog
-					next if $flds[$idx] eq 'n.nspname' and ! DBD::Pg::_pg_check_version(7.3, $version);
-					if(defined $attrs[$idx] and length $attrs[$idx]) {
-						# Change the "name" of the types to the real value.
-						if ($flds[$idx] =~ m/relkind/) {
-							$attrs[$idx] =~ s/^\'?table\'?/'r'/i;
-							$attrs[$idx] =~ s/^\'?index\'?/'i'/i;
-							$attrs[$idx] =~ s/^\'?sequence\'?/'S'/i;
-							$attrs[$idx] =~ s/^\'?view\'?/'v'/i;
-							$attrs[$idx] =~ s/^\'?special\'?/'s'/i;
-							$attrs[$idx] =~ s/^\'?secondary\'?/'t'/i;
-						}
-						# Insure that the value is enclosed in single quotes.
-						$attrs[$idx] =~ s/^'?(\w+)'?$/'$1'/;
-						if ($attrs[$idx] =~ m/[,%]/) {
-							# contains a meta character.
-							push( @wh, q{( } . join ( " OR "
-								, map { m/\%/ 
-									? qq{$flds[$idx] LIKE $_ }
-									: qq{$flds[$idx] = $_ }
-									} (split /,/, $attrs[$idx]) )
-									. q{ )}
-								);
-						}
-						else {
-							push( @wh, qq{$flds[$idx] = $attrs[$idx]} );
-						}
-					}
-				}
-
-				my $wh = ();
-				if (@wh) {
-					$wh = join( " AND ",'', @wh );
-					$tbl_sql = qq{
-					SELECT NULL::text  AS "TABLE_CAT"
-						 , $showschema   AS "TABLE_SCHEM"
-						 , c.relname     AS "TABLE_NAME"
-						 , CASE
-								WHEN c.relkind = 'r' THEN
-								$schemacase
-								WHEN c.relkind = 'v' THEN 'VIEW'
-								WHEN c.relkind = 'i' THEN 'INDEX'
-								WHEN c.relkind = 'S' THEN 'SEQUENCE'
-								WHEN c.relkind = 's' THEN 'SPECIAL'
-								WHEN c.relkind = 't' THEN 'SECONDARY'
-								ELSE 'UNKNOWN'
-							END	AS "TABLE_TYPE"
-						 , d.description AS "REMARKS"
-					FROM ${CATALOG}pg_class AS c
-						LEFT JOIN
-						${CATALOG}pg_description AS d 
-							ON (c.relfilenode = d.objoid $has_objsubid)
-						$schemajoin
-					WHERE 
-						c.relname !~ '^xin[vx][0-9]+'
-					$wh
-					ORDER BY 2, 3
-					};
-				}
-			}
 		}
-
 		my $sth = $dbh->prepare( $tbl_sql ) or return undef;
 		$sth->execute();
 
 		return $sth;
 	}
 
-
-	sub tables {
-		my($dbh) = @_;
-		my $version = DBD::Pg::_pg_server_version($dbh);
-
-		my $SQL = DBD::Pg::_pg_check_version(7.3, $version) ? 
-			"SELECT n.nspname AS \"SCHEMA_NAME\", c.relname AS \"TABLE_NAME\"
-			FROM pg_catalog.pg_class c
-			LEFT JOIN pg_catalog.pg_namespace n ON (n.oid = c.relnamespace)
-			WHERE c.relkind = 'r'
-			AND n.nspname NOT IN ('pg_catalog', 'pg_toast')
-			AND pg_catalog.pg_table_is_visible(c.oid)
-			ORDER BY 1,2"
-			:
-			"SELECT relname AS \"TABLE_NAME\"
-			FROM   pg_class 
-			WHERE  relkind = 'r'
-			AND    relname !~ '^pg_'
-			AND    relname !~ '^xin[vx][0-9]+'
-			ORDER BY 1";
-		my $sth = $dbh->prepare($SQL) or return undef;
-		$sth->execute or return undef;
-		my (@tables, @relname);
-		while (@relname = $sth->fetchrow_array) {
-			push @tables, DBD::Pg::_pg_check_version(7.3, $version) ? 
-			"$relname[0].$relname[1]" : $relname[0];
-		}
-		$sth->finish;
-		return @tables;
+  sub tables {
+			my ($dbh, @args) = @_;
+			my $sth = $dbh->table_info(@args) or return;
+			my $tables = $sth->fetchall_arrayref() or return;
+			my $version = DBD::Pg::_pg_server_version($dbh);
+			my @tables = map { DBD::Pg::_pg_check_version(7.3, $version) ? 
+														 "$_->[1].$_->[2]" : $_->[2] } @$tables;
+			return @tables;
 	}
 
 	sub table_attributes {
@@ -1290,12 +1124,6 @@ $DBD::Pg::VERSION = '1.31_5';
 
 }
 
-
-# Ummm, I don't think this extra curly brace should be here,
-# but it quite complaining when I added and things to seem to be working.
-# I'm worried. -mls 07/22/03 
-# }
-
 1;
 
 __END__
@@ -1715,7 +1543,6 @@ Supported by the driver as proposed by the DBI with the follow exceptions.
 These fields are currently always returned with NULL values:
 
    TABLE_CAT
-   TYPE_NAME
    BUFFER_LENGTH
    DECIMAL_DIGITS
    NUM_PREC_RADIX
@@ -1723,38 +1550,24 @@ These fields are currently always returned with NULL values:
    SQL_DATETIME_SUB
    CHAR_OCTET_LENGTH
 
-Also, one additional non-standard field is returned:
-
-  pg_constraint - holds column constraint definition
-
-=item B<column_info>
-
-	$sth = $dbh->column_info( $catalog, $schema, $table, $column );
-
-Supported by the driver as proposed by the DBI with the follow exceptions.
-These fields are currently always returned with NULL values:
-
-   TABLE_CAT
-   TYPE_NAME
-   BUFFER_LENGTH
-   DECIMAL_DIGITS
-   NUM_PREC_RADIX
-   SQL_DATA_TYPE
-   SQL_DATETIME_SUB
-   CHAR_OCTET_LENGTH
-
-Also, one additional non-standard field is returned:
+Also, four additional non-standard fields are returned:
 
+  pg_type
+  pg_type_only
+  pg_attypmod
   pg_constraint - holds column constraint definition
 
 =item B<table_info>
 
-  $sth = $dbh->table_info;
+  $sth = $dbh->table_info( $catalog, $schema, $table, $type );
+
+Supported by the driver as proposed by DBI. This method returns all tables 
+and views visible to the current user. The $catalog argument is currently 
+unused. The schema and table arguments will do a 'LIKE' search if a 
+percent sign (%) or an underscore (_) are detected in the argument.
+The $type argument accepts a value of wither "TABLE" or "VIEW" 
+(using both is the default action).
 
-Supported by the driver as proposed by DBI. This method returns all table.
-which are owned by the current user. It does not select any indexes and
-sequences. Also System tables are not selected. As TABLE_QUALIFIER the reltype
-attribute is returned and the REMARKS are undefined.
 
 =item B<foreign_key_info>
 
@@ -1767,11 +1580,13 @@ about first column of any multiple-column keys.
 
 =item B<tables>
 
-  @names = $dbh->tables;
+  @names = $dbh->tables( $catalog, $schema, $table, $type );
 
 Supported by the driver as proposed by DBI. This method returns all tables and
-views which are owned by the current user. It does not select any indexes and
-sequences, or system tables.
+views which are visible to the current user. If the database is version 7.3 
+or higher, the name of the schema appears before the table or view name.
+
+
 
 =item B<type_info_all>
 
