@@ -57,8 +57,8 @@ static sql_type_info_t pg_types[] = {
 	{VARCHAROID, "varchar", DBDPG_TRUE, quote_varchar, dequote_varchar, {SQL_VARCHAR}},
 	{DATEOID, "date", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{TIMEOID, "time", DBDPG_TRUE, null_quote, null_dequote, {0}},
-	{TIMESTAMPOID, "timestamp", DBDPG_TRUE, null_quote, null_dequote, {SQL_TYPE_TIMESTAMP}},
-	{TIMESTAMPTZOID, "datetime", DBDPG_TRUE, null_quote, null_dequote, {SQL_TYPE_TIMESTAMP_WITH_TIMEZONE}},
+	{TIMESTAMPOID, "timestamp", DBDPG_TRUE, null_quote, null_dequote, {0}},
+	{TIMESTAMPTZOID, "datetime", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{INTERVALOID, "timespan", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{TIMETZOID, "timestamptz", DBDPG_TRUE, null_quote, null_dequote, {0}},
 	{BITOID, "bitstring", DBDPG_TRUE, null_quote, null_dequote, {0}},
